@@ -327,7 +327,7 @@ static HRESULT exprval_call(script_ctx_t *ctx, exprval_t *ref, WORD flags, unsig
             return E_FAIL;
         }
 
-        return disp_call_value(ctx, get_object(v), NULL, flags, argc, argv, r);
+        return disp_call_value(ctx, get_object(v), jsval_undefined(), flags, argc, argv, r);
     }
     case EXPRVAL_IDREF:
         /* ECMA-262 3rd Edition 11.2.3.7 / ECMA-262 5.1 Edition 11.2.3.6 *
@@ -340,7 +340,7 @@ static HRESULT exprval_call(script_ctx_t *ctx, exprval_t *ref, WORD flags, unsig
                 FIXME("invoke %s\n", debugstr_jsval(v));
                 hres = E_FAIL;
             }else {
-                hres = disp_call_value(ctx, get_object(v), NULL, flags, argc, argv, r);
+                hres = disp_call_value(ctx, get_object(v), jsval_undefined(), flags, argc, argv, r);
             }
             jsval_release(v);
             return hres;
@@ -351,7 +351,7 @@ static HRESULT exprval_call(script_ctx_t *ctx, exprval_t *ref, WORD flags, unsig
 
         hres = to_object(ctx, ref->u.val, &obj);
         if(SUCCEEDED(hres)) {
-            hres = disp_call_value(ctx, obj, NULL, flags, argc, argv, r);
+            hres = disp_call_value(ctx, obj, jsval_undefined(), flags, argc, argv, r);
             IDispatch_Release(obj);
         }
         return hres;
@@ -426,7 +426,7 @@ static HRESULT scope_push(scope_chain_t *scope, jsdisp_t *jsobj, IDispatch *obj,
 {
     scope_chain_t *new_scope;
 
-    new_scope = heap_alloc(sizeof(scope_chain_t));
+    new_scope = malloc(sizeof(scope_chain_t));
     if(!new_scope)
         return E_OUTOFMEMORY;
 
@@ -463,7 +463,7 @@ void scope_release(scope_chain_t *scope)
 
     if (scope->obj)
         IDispatch_Release(scope->obj);
-    heap_free(scope);
+    free(scope);
 }
 
 static HRESULT disp_get_id(script_ctx_t *ctx, IDispatch *disp, const WCHAR *name, BSTR name_bstr, DWORD flags, DISPID *id)
@@ -1115,7 +1115,7 @@ static HRESULT interp_push_except(script_ctx_t *ctx)
 
     TRACE("\n");
 
-    except = heap_alloc(sizeof(*except));
+    except = malloc(sizeof(*except));
     if(!except)
         return E_OUTOFMEMORY;
 
@@ -1143,7 +1143,7 @@ static HRESULT interp_pop_except(script_ctx_t *ctx)
 
     finally_off = except->finally_off;
     frame->except_frame = except->next;
-    heap_free(except);
+    free(except);
 
     if(finally_off) {
         HRESULT hres;
@@ -1380,7 +1380,7 @@ static HRESULT interp_new(script_ctx_t *ctx)
         return JS_E_INVALID_ACTION;
 
     clear_acc(ctx);
-    return disp_call_value(ctx, get_object(constr), NULL, DISPATCH_CONSTRUCT | DISPATCH_JSCRIPT_CALLEREXECSSOURCE,
+    return disp_call_value(ctx, get_object(constr), jsval_undefined(), DISPATCH_CONSTRUCT | DISPATCH_JSCRIPT_CALLEREXECSSOURCE,
                            argc, stack_args(ctx, argc), &ctx->acc);
 }
 
@@ -1398,7 +1398,7 @@ static HRESULT interp_call(script_ctx_t *ctx)
         return JS_E_INVALID_PROPERTY;
 
     clear_acc(ctx);
-    return disp_call_value(ctx, get_object(obj), NULL, DISPATCH_METHOD | DISPATCH_JSCRIPT_CALLEREXECSSOURCE,
+    return disp_call_value(ctx, get_object(obj), jsval_undefined(), DISPATCH_METHOD | DISPATCH_JSCRIPT_CALLEREXECSSOURCE,
                            argn, stack_args(ctx, argn), do_ret ? &ctx->acc : NULL);
 }
 
@@ -2944,7 +2944,7 @@ static void pop_call_frame(script_ctx_t *ctx)
         IDispatch_Release(frame->this_obj);
     jsval_release(frame->ret);
     release_bytecode(frame->bytecode);
-    heap_free(frame);
+    free(frame);
 }
 
 static void print_backtrace(script_ctx_t *ctx)
@@ -3056,7 +3056,7 @@ static HRESULT unwind_exception(script_ctx_t *ctx, HRESULT exception_hres)
         except_frame->catch_off = 0;
     }else {
         frame->except_frame = except_frame->next;
-        heap_free(except_frame);
+        free(except_frame);
     }
 
     hres = stack_push(ctx, except_val);
@@ -3231,7 +3231,7 @@ HRESULT exec_source(script_ctx_t *ctx, DWORD flags, bytecode_t *bytecode, functi
     HRESULT hres;
 
     if(!ctx->stack) {
-        ctx->stack = heap_alloc(stack_size * sizeof(*ctx->stack));
+        ctx->stack = malloc(stack_size * sizeof(*ctx->stack));
         if(!ctx->stack)
             return E_OUTOFMEMORY;
     }
@@ -3331,7 +3331,7 @@ HRESULT exec_source(script_ctx_t *ctx, DWORD flags, bytecode_t *bytecode, functi
             goto fail;
     }
 
-    frame = heap_alloc_zero(sizeof(*frame));
+    frame = calloc(1, sizeof(*frame));
     if(!frame) {
         hres = E_OUTOFMEMORY;
         goto fail;
@@ -3346,7 +3346,7 @@ HRESULT exec_source(script_ctx_t *ctx, DWORD flags, bytecode_t *bytecode, functi
         hres = setup_scope(ctx, frame, scope, variable_obj, argc, argv);
         if(FAILED(hres)) {
             release_bytecode(frame->bytecode);
-            heap_free(frame);
+            free(frame);
             goto fail;
         }
     }else if(scope) {
