@@ -134,7 +134,7 @@ static void release_map_entry(struct jsval_map_entry *entry)
     jsval_release(entry->key);
     jsval_release(entry->value);
     list_remove(&entry->list_entry);
-    heap_free(entry);
+    free(entry);
 }
 
 static void delete_map_entry(MapInstance *map, struct jsval_map_entry *entry)
@@ -159,7 +159,7 @@ static HRESULT set_map_entry(MapInstance *map, jsval_t key, jsval_t value, jsval
         jsval_release(entry->value);
         entry->value = val;
     }else {
-        if(!(entry = heap_alloc_zero(sizeof(*entry)))) return E_OUTOFMEMORY;
+        if(!(entry = calloc(1, sizeof(*entry)))) return E_OUTOFMEMORY;
 
         hres = jsval_copy(key, &entry->key);
         if(SUCCEEDED(hres)) {
@@ -168,7 +168,7 @@ static HRESULT set_map_entry(MapInstance *map, jsval_t key, jsval_t value, jsval
                 jsval_release(entry->key);
         }
         if(FAILED(hres)) {
-            heap_free(entry);
+            free(entry);
             return hres;
         }
         grab_map_entry(entry);
@@ -184,7 +184,7 @@ static HRESULT set_map_entry(MapInstance *map, jsval_t key, jsval_t value, jsval
 static HRESULT iterate_map(MapInstance *map, script_ctx_t *ctx, unsigned argc, jsval_t *argv, jsval_t *r)
 {
     struct list *iter = list_head(&map->entries);
-    IDispatch *context_obj = NULL;
+    jsval_t context_this = jsval_undefined();
     HRESULT hres;
 
     if(!argc || !is_object_instance(argv[0])) {
@@ -192,13 +192,8 @@ static HRESULT iterate_map(MapInstance *map, script_ctx_t *ctx, unsigned argc, j
         return E_FAIL;
     }
 
-    if(argc > 1 && !is_undefined(argv[1])) {
-        if(!is_object_instance(argv[1])) {
-            FIXME("Unsupported context this %s\n", debugstr_jsval(argv[1]));
-            return E_NOTIMPL;
-        }
-        context_obj = get_object(argv[1]);
-    }
+    if(argc > 1)
+        context_this = argv[1];
 
     while(iter) {
         struct jsval_map_entry *entry = LIST_ENTRY(iter, struct jsval_map_entry, list_entry);
@@ -213,8 +208,7 @@ static HRESULT iterate_map(MapInstance *map, script_ctx_t *ctx, unsigned argc, j
         args[1] = entry->key;
         args[2] = jsval_obj(&map->dispex);
         grab_map_entry(entry);
-        hres = disp_call_value(ctx, get_object(argv[0]), context_obj,
-                               DISPATCH_METHOD, ARRAY_SIZE(args), args, &v);
+        hres = disp_call_value(ctx, get_object(argv[0]), context_this, DISPATCH_METHOD, ARRAY_SIZE(args), args, &v);
         iter = list_next(&map->entries, iter);
         release_map_entry(entry);
         if(FAILED(hres))
@@ -366,7 +360,7 @@ static void Map_destructor(jsdisp_t *dispex)
         release_map_entry(entry);
     }
 
-    heap_free(map);
+    free(map);
 }
 static const builtin_prop_t Map_prototype_props[] = {
     {L"clear",      Map_clear,     PROPF_METHOD},
@@ -411,7 +405,7 @@ static HRESULT Map_constructor(script_ctx_t *ctx, jsval_t vthis, WORD flags, uns
 
         if(!r)
             return S_OK;
-        if(!(map = heap_alloc_zero(sizeof(*map))))
+        if(!(map = calloc(1, sizeof(*map))))
             return E_OUTOFMEMORY;
 
         hres = init_dispex(&map->dispex, ctx, &Map_info, ctx->map_prototype);
@@ -423,6 +417,9 @@ static HRESULT Map_constructor(script_ctx_t *ctx, jsval_t vthis, WORD flags, uns
         *r = jsval_obj(&map->dispex);
         return S_OK;
 
+    case DISPATCH_METHOD:
+        return throw_error(ctx, JS_E_WRONG_THIS, L"Map");
+
     default:
         FIXME("unimplemented flags %x\n", flags);
         return E_NOTIMPL;
@@ -563,7 +560,7 @@ static HRESULT Set_constructor(script_ctx_t *ctx, jsval_t vthis, WORD flags, uns
 
         if(!r)
             return S_OK;
-        if(!(set = heap_alloc_zero(sizeof(*set))))
+        if(!(set = calloc(1, sizeof(*set))))
             return E_OUTOFMEMORY;
 
         hres = init_dispex(&set->dispex, ctx, &Set_info, ctx->set_prototype);
@@ -575,6 +572,9 @@ static HRESULT Set_constructor(script_ctx_t *ctx, jsval_t vthis, WORD flags, uns
         *r = jsval_obj(&set->dispex);
         return S_OK;
 
+    case DISPATCH_METHOD:
+        return throw_error(ctx, JS_E_WRONG_THIS, L"Set");
+
     default:
         FIXME("unimplemented flags %x\n", flags);
         return E_NOTIMPL;
