@@ -600,6 +600,9 @@ static DBusMessage *device_by_iface_request( const char *iface )
     DBusMessage *request, *reply;
     DBusMessageIter iter;
     DBusError error;
+    DBusConnection *connection = get_dhcp_connection();
+
+    if (!connection) return NULL;
 
     request = p_dbus_message_new_method_call( "org.freedesktop.NetworkManager", "/org/freedesktop/NetworkManager",
                                               "org.freedesktop.NetworkManager", "GetDeviceByIpIface" );
@@ -609,7 +612,7 @@ static DBusMessage *device_by_iface_request( const char *iface )
     p_dbus_message_iter_append_basic( &iter, DBUS_TYPE_STRING, &iface );
 
     p_dbus_error_init( &error );
-    reply = p_dbus_connection_send_with_reply_and_block( get_dhcp_connection(), request, -1, &error );
+    reply = p_dbus_connection_send_with_reply_and_block( connection, request, -1, &error );
     p_dbus_message_unref( request );
     if (!reply)
     {
@@ -777,8 +780,9 @@ NTSTATUS dhcp_request( void *args )
     ULONG ret = 0;
 
     params->req->offset = params->req->size = 0;
+    *params->ret_size = 0;
 
-    if (!(reply = dhcp4_config_option_request( params->unix_name, map_option(params->req->id), &value ))) return 0;
+    if (!(reply = dhcp4_config_option_request( params->unix_name, map_option(params->req->id), &value ))) return STATUS_SUCCESS;
 
     switch (params->req->id)
     {
