@@ -489,7 +489,7 @@ HRESULT HTMLAreaElement_Create(HTMLDocumentNode *doc, nsIDOMElement *nselem, HTM
     HTMLAreaElement *ret;
     nsresult nsres;
 
-    ret = heap_alloc_zero(sizeof(HTMLAreaElement));
+    ret = calloc(1, sizeof(HTMLAreaElement));
     if(!ret)
         return E_OUTOFMEMORY;
 
