@@ -124,7 +124,7 @@ static ULONG WINAPI HTMLLocation_Release(IHTMLLocation *iface)
         if(This->window)
             This->window->location = NULL;
         release_dispex(&This->dispex);
-        heap_free(This);
+        free(This);
     }
 
     return ref;
@@ -201,7 +201,7 @@ static HRESULT WINAPI HTMLLocation_get_href(IHTMLLocation *iface, BSTR *p)
     case INTERNET_SCHEME_FILE:
         {
             /* prepend a slash */
-            url_path = HeapAlloc(GetProcessHeap(), 0, (url.dwUrlPathLength + 1) * sizeof(WCHAR));
+            url_path = malloc((url.dwUrlPathLength + 1) * sizeof(WCHAR));
             if(!url_path)
                 return E_OUTOFMEMORY;
             url_path[0] = '/';
@@ -216,7 +216,7 @@ static HRESULT WINAPI HTMLLocation_get_href(IHTMLLocation *iface, BSTR *p)
     case INTERNET_SCHEME_FTP:
         if(!url.dwUrlPathLength) {
             /* add a slash if it's blank */
-            url_path = url.lpszUrlPath = HeapAlloc(GetProcessHeap(), 0, 1 * sizeof(WCHAR));
+            url_path = url.lpszUrlPath = malloc(sizeof(WCHAR));
             if(!url.lpszUrlPath)
                 return E_OUTOFMEMORY;
             url.lpszUrlPath[0] = '/';
@@ -247,7 +247,7 @@ static HRESULT WINAPI HTMLLocation_get_href(IHTMLLocation *iface, BSTR *p)
     }
     SetLastError(0);
 
-    buf = HeapAlloc(GetProcessHeap(), 0, len * sizeof(WCHAR));
+    buf = malloc(len * sizeof(WCHAR));
     if(!buf) {
         ret = E_OUTOFMEMORY;
         goto cleanup;
@@ -269,8 +269,8 @@ static HRESULT WINAPI HTMLLocation_get_href(IHTMLLocation *iface, BSTR *p)
     ret = S_OK;
 
 cleanup:
-    HeapFree(GetProcessHeap(), 0, buf);
-    HeapFree(GetProcessHeap(), 0, url_path);
+    free(buf);
+    free(url_path);
 
     return ret;
 }
@@ -536,7 +536,7 @@ static HRESULT WINAPI HTMLLocation_put_hash(IHTMLLocation *iface, BSTR v)
 
     if(hash[0] != '#') {
         unsigned size = (1 /* # */ + wcslen(v) + 1) * sizeof(WCHAR);
-        if(!(hash = heap_alloc(size)))
+        if(!(hash = malloc(size)))
             return E_OUTOFMEMORY;
         hash[0] = '#';
         memcpy(hash + 1, v, size - sizeof(WCHAR));
@@ -545,7 +545,7 @@ static HRESULT WINAPI HTMLLocation_put_hash(IHTMLLocation *iface, BSTR v)
     hres = navigate_url(This->window->base.outer_window, hash, This->window->base.outer_window->uri, BINDING_NAVIGATED);
 
     if(hash != v)
-        heap_free(hash);
+        free(hash);
     return hres;
 }
 
@@ -663,7 +663,7 @@ HRESULT HTMLLocation_Create(HTMLInnerWindow *window, HTMLLocation **ret)
 {
     HTMLLocation *location;
 
-    location = heap_alloc(sizeof(*location));
+    location = malloc(sizeof(*location));
     if(!location)
         return E_OUTOFMEMORY;
 
