@@ -43,8 +43,8 @@ static const WORD current_machine = IMAGE_FILE_MACHINE_ARM64;
 extern WORD native_machine DECLSPEC_HIDDEN;
 extern HMODULE ntdll_module DECLSPEC_HIDDEN;
 
-extern const unixlib_entry_t __wine_unix_call_funcs[];
-extern const unixlib_entry_t __wine_unix_call_wow64_funcs[];
+extern const unixlib_entry_t __wine_unix_call_funcs[] DECLSPEC_HIDDEN;
+extern const unixlib_entry_t __wine_unix_call_wow64_funcs[] DECLSPEC_HIDDEN;
 
 static const BOOL is_win64 = (sizeof(void *) > sizeof(int));
 
@@ -76,7 +76,7 @@ static inline struct ntdll_thread_data *ntdll_get_thread_data(void)
 }
 
 /* returns TRUE if the async is complete; FALSE if it should be restarted */
-typedef BOOL async_callback_t( void *user, ULONG_PTR *info, NTSTATUS *status );
+typedef BOOL async_callback_t( void *user, ULONG_PTR *info, unsigned int *status );
 
 struct async_fileio
 {
@@ -190,8 +190,8 @@ extern void wait_suspend( CONTEXT *context ) DECLSPEC_HIDDEN;
 extern NTSTATUS send_debug_event( EXCEPTION_RECORD *rec, CONTEXT *context, BOOL first_chance ) DECLSPEC_HIDDEN;
 extern NTSTATUS set_thread_context( HANDLE handle, const void *context, BOOL *self, USHORT machine ) DECLSPEC_HIDDEN;
 extern NTSTATUS get_thread_context( HANDLE handle, void *context, BOOL *self, USHORT machine ) DECLSPEC_HIDDEN;
-extern NTSTATUS alloc_object_attributes( const OBJECT_ATTRIBUTES *attr, struct object_attributes **ret,
-                                         data_size_t *ret_len ) DECLSPEC_HIDDEN;
+extern unsigned int alloc_object_attributes( const OBJECT_ATTRIBUTES *attr, struct object_attributes **ret,
+                                             data_size_t *ret_len ) DECLSPEC_HIDDEN;
 extern NTSTATUS system_time_precise( void *args ) DECLSPEC_HIDDEN;
 
 extern void *anon_mmap_fixed( void *start, size_t size, int prot, int flags ) DECLSPEC_HIDDEN;
@@ -252,21 +252,21 @@ extern void fill_vm_counters( VM_COUNTERS_EX *pvmi, int unix_pid ) DECLSPEC_HIDD
 extern NTSTATUS open_hkcu_key( const char *path, HANDLE *key ) DECLSPEC_HIDDEN;
 
 extern NTSTATUS cdrom_DeviceIoControl( HANDLE device, HANDLE event, PIO_APC_ROUTINE apc, void *apc_user,
-                                       IO_STATUS_BLOCK *io, ULONG code, void *in_buffer,
-                                       ULONG in_size, void *out_buffer, ULONG out_size ) DECLSPEC_HIDDEN;
+                                       IO_STATUS_BLOCK *io, UINT code, void *in_buffer,
+                                       UINT in_size, void *out_buffer, UINT out_size ) DECLSPEC_HIDDEN;
 extern NTSTATUS serial_DeviceIoControl( HANDLE device, HANDLE event, PIO_APC_ROUTINE apc, void *apc_user,
-                                        IO_STATUS_BLOCK *io, ULONG code, void *in_buffer,
-                                        ULONG in_size, void *out_buffer, ULONG out_size ) DECLSPEC_HIDDEN;
+                                        IO_STATUS_BLOCK *io, UINT code, void *in_buffer,
+                                        UINT in_size, void *out_buffer, UINT out_size ) DECLSPEC_HIDDEN;
 extern NTSTATUS serial_FlushBuffersFile( int fd ) DECLSPEC_HIDDEN;
 extern NTSTATUS sock_ioctl( HANDLE handle, HANDLE event, PIO_APC_ROUTINE apc, void *apc_user, IO_STATUS_BLOCK *io,
-                            ULONG code, void *in_buffer, ULONG in_size, void *out_buffer, ULONG out_size ) DECLSPEC_HIDDEN;
+                            UINT code, void *in_buffer, UINT in_size, void *out_buffer, UINT out_size ) DECLSPEC_HIDDEN;
 extern NTSTATUS sock_read( HANDLE handle, int fd, HANDLE event, PIO_APC_ROUTINE apc, void *apc_user,
                            IO_STATUS_BLOCK *io, void *buffer, ULONG length ) DECLSPEC_HIDDEN;
 extern NTSTATUS sock_write( HANDLE handle, int fd, HANDLE event, PIO_APC_ROUTINE apc, void *apc_user,
                             IO_STATUS_BLOCK *io, const void *buffer, ULONG length ) DECLSPEC_HIDDEN;
 extern NTSTATUS tape_DeviceIoControl( HANDLE device, HANDLE event, PIO_APC_ROUTINE apc, void *apc_user,
-                                      IO_STATUS_BLOCK *io, ULONG code, void *in_buffer,
-                                      ULONG in_size, void *out_buffer, ULONG out_size ) DECLSPEC_HIDDEN;
+                                      IO_STATUS_BLOCK *io, UINT code, void *in_buffer,
+                                      UINT in_size, void *out_buffer, UINT out_size ) DECLSPEC_HIDDEN;
 
 extern struct async_fileio *alloc_fileio( DWORD size, async_callback_t callback, HANDLE handle ) DECLSPEC_HIDDEN;
 extern void release_fileio( struct async_fileio *io ) DECLSPEC_HIDDEN;
@@ -367,6 +367,24 @@ static inline void set_async_iosb( client_ptr_t iosb, NTSTATUS status, ULONG_PTR
 {
     if (!iosb) return;
 
+    /* GetOverlappedResult() and WSAGetOverlappedResult() expect that if the
+     * status is written, that the information (and buffer, which was written
+     * earlier from the async callback) will be available. Hence we need to
+     * store the status last, with release semantics to ensure that those
+     * writes are visible. This release is paired with a read-acquire in
+     * GetOverlappedResult() and WSAGetOverlappedResult():
+     *
+     * CPU 0 (set_async_iosb)            CPU 1 (GetOverlappedResultEx)
+     * ===========================       ===========================
+     * write buffer
+     * write Information
+     * WriteRelease(Status) <--------.
+     *                               |
+     *                               |
+     *                (paired with)  `-> ReadAcquire(Status)
+     *                                   read Information
+     */
+
     if (in_wow64_call())
     {
         struct iosb32
@@ -374,18 +392,18 @@ static inline void set_async_iosb( client_ptr_t iosb, NTSTATUS status, ULONG_PTR
             NTSTATUS Status;
             ULONG    Information;
         } *io = wine_server_get_ptr( iosb );
-        io->Status = status;
         io->Information = info;
+        WriteRelease( &io->Status, status );
     }
     else
     {
         IO_STATUS_BLOCK *io = wine_server_get_ptr( iosb );
-#ifdef NONAMELESSUNION
-        io->u.Status = status;
-#else
-        io->Status = status;
-#endif
         io->Information = info;
+#ifdef NONAMELESSUNION
+        WriteRelease( &io->u.Status, status );
+#else
+        WriteRelease( &io->Status, status );
+#endif
     }
 }
 
