@@ -267,9 +267,9 @@ static HRESULT add_dynamic_var(exec_ctx_t *ctx, const WCHAR *name,
         if(cnt > script_obj->global_vars_size) {
             dynamic_var_t **new_vars;
             if(script_obj->global_vars)
-                new_vars = heap_realloc(script_obj->global_vars, cnt * 2 * sizeof(*new_vars));
+                new_vars = realloc(script_obj->global_vars, cnt * 2 * sizeof(*new_vars));
             else
-                new_vars = heap_alloc(cnt * 2 * sizeof(*new_vars));
+                new_vars = malloc(cnt * 2 * sizeof(*new_vars));
             if(!new_vars)
                 return E_OUTOFMEMORY;
             script_obj->global_vars = new_vars;
@@ -318,7 +318,7 @@ static HRESULT stack_push(exec_ctx_t *ctx, VARIANT *v)
     if(ctx->stack_size == ctx->top) {
         VARIANT *new_stack;
 
-        new_stack = heap_realloc(ctx->stack, ctx->stack_size*2*sizeof(*ctx->stack));
+        new_stack = realloc(ctx->stack, ctx->stack_size*2*sizeof(*ctx->stack));
         if(!new_stack) {
             VariantClear(v);
             return E_OUTOFMEMORY;
@@ -535,7 +535,7 @@ static void vbstack_to_dp(exec_ctx_t *ctx, unsigned arg_cnt, BOOL is_propput, DI
     }
 }
 
-static HRESULT array_access(exec_ctx_t *ctx, SAFEARRAY *array, DISPPARAMS *dp, VARIANT **ret)
+HRESULT array_access(SAFEARRAY *array, DISPPARAMS *dp, VARIANT **ret)
 {
     unsigned i, argc = arg_cnt(dp);
     LONG *indices;
@@ -556,7 +556,7 @@ static HRESULT array_access(exec_ctx_t *ctx, SAFEARRAY *array, DISPPARAMS *dp, V
         return E_FAIL;
     }
 
-    indices = heap_alloc(sizeof(*indices) * argc);
+    indices = malloc(sizeof(*indices) * argc);
     if(!indices) {
         SafeArrayUnlock(array);
         return E_OUTOFMEMORY;
@@ -565,7 +565,7 @@ static HRESULT array_access(exec_ctx_t *ctx, SAFEARRAY *array, DISPPARAMS *dp, V
     for(i=0; i<argc; i++) {
         hres = to_int(get_arg(dp, i), (int *)(indices+i));
         if(FAILED(hres)) {
-            heap_free(indices);
+            free(indices);
             SafeArrayUnlock(array);
             return hres;
         }
@@ -573,7 +573,7 @@ static HRESULT array_access(exec_ctx_t *ctx, SAFEARRAY *array, DISPPARAMS *dp, V
 
     hres = SafeArrayPtrOfIndex(array, indices, (void**)ret);
     SafeArrayUnlock(array);
-    heap_free(indices);
+    free(indices);
     return hres;
 }
 
@@ -611,7 +611,7 @@ static HRESULT variant_call(exec_ctx_t *ctx, VARIANT *v, unsigned arg_cnt, VARIA
         }
 
         vbstack_to_dp(ctx, arg_cnt, FALSE, &dp);
-        hres = array_access(ctx, array, &dp, &v);
+        hres = array_access(array, &dp, &v);
         if(FAILED(hres))
             return hres;
 
@@ -894,7 +894,7 @@ static HRESULT assign_ident(exec_ctx_t *ctx, BSTR name, WORD flags, DISPPARAMS *
                 return E_FAIL;
             }
 
-            hres = array_access(ctx, array, dp, &v);
+            hres = array_access(array, dp, &v);
             if(FAILED(hres))
                 return hres;
         }else if(V_VT(v) == (VT_ARRAY|VT_BYREF|VT_VARIANT)) {
@@ -1216,7 +1216,7 @@ static HRESULT interp_dim(exec_ctx_t *ctx)
         ref_t ref;
 
         if(!ctx->arrays) {
-            ctx->arrays = heap_alloc_zero(ctx->func->array_cnt * sizeof(SAFEARRAY*));
+            ctx->arrays = calloc(ctx->func->array_cnt, sizeof(SAFEARRAY*));
             if(!ctx->arrays)
                 return E_OUTOFMEMORY;
         }
@@ -1260,13 +1260,13 @@ static HRESULT array_bounds_from_stack(exec_ctx_t *ctx, unsigned dim_cnt, SAFEAR
     int dim;
     HRESULT hres;
 
-    if(!(bounds = heap_alloc(dim_cnt * sizeof(*bounds))))
+    if(!(bounds = malloc(dim_cnt * sizeof(*bounds))))
         return E_OUTOFMEMORY;
 
     for(i = 0; i < dim_cnt; i++) {
         hres = to_int(stack_top(ctx, dim_cnt - i - 1), &dim);
         if(FAILED(hres)) {
-            heap_free(bounds);
+            free(bounds);
             return hres;
         }
 
@@ -1283,6 +1283,7 @@ static HRESULT interp_redim(exec_ctx_t *ctx)
 {
     BSTR identifier = ctx->instr->arg1.bstr;
     const unsigned dim_cnt = ctx->instr->arg2.uint;
+    VARIANT *v;
     SAFEARRAYBOUND *bounds;
     SAFEARRAY *array;
     ref_t ref;
@@ -1306,15 +1307,22 @@ static HRESULT interp_redim(exec_ctx_t *ctx)
         return hres;
 
     array = SafeArrayCreate(VT_VARIANT, dim_cnt, bounds);
-    heap_free(bounds);
+    free(bounds);
     if(!array)
         return E_OUTOFMEMORY;
 
     /* FIXME: We should check if we're not modifying an existing static array here */
 
-    VariantClear(ref.u.v);
-    V_VT(ref.u.v) = VT_ARRAY|VT_VARIANT;
-    V_ARRAY(ref.u.v) = array;
+    v = ref.u.v;
+
+    if(V_VT(v) == (VT_VARIANT|VT_BYREF)) {
+        v = V_VARIANTREF(v);
+    }
+
+    VariantClear(v);
+    V_VT(v) = VT_ARRAY|VT_VARIANT;
+    V_ARRAY(v) = array;
+
     return S_OK;
 }
 
@@ -1323,6 +1331,7 @@ static HRESULT interp_redim_preserve(exec_ctx_t *ctx)
     BSTR identifier = ctx->instr->arg1.bstr;
     const unsigned dim_cnt = ctx->instr->arg2.uint;
     unsigned i;
+    VARIANT *v;
     SAFEARRAYBOUND *bounds;
     SAFEARRAY *array;
     ref_t ref;
@@ -1341,12 +1350,18 @@ static HRESULT interp_redim_preserve(exec_ctx_t *ctx)
         return E_FAIL;
     }
 
-    if(!(V_VT(ref.u.v) & VT_ARRAY)) {
-        FIXME("ReDim Preserve not valid on type %d\n", V_VT(ref.u.v));
+    v = ref.u.v;
+
+    if(V_VT(v) == (VT_VARIANT|VT_BYREF)) {
+        v = V_VARIANTREF(v);
+    }
+
+    if(!(V_VT(v) & VT_ARRAY)) {
+        FIXME("ReDim Preserve not valid on type %d\n", V_VT(v));
         return E_FAIL;
     }
 
-    array = V_ARRAY(ref.u.v);
+    array = V_ARRAY(v);
 
     hres = array_bounds_from_stack(ctx, dim_cnt, &bounds);
     if(FAILED(hres))
@@ -1355,9 +1370,9 @@ static HRESULT interp_redim_preserve(exec_ctx_t *ctx)
     if(array == NULL || array->cDims == 0) {
         /* can initially allocate the array */
         array = SafeArrayCreate(VT_VARIANT, dim_cnt, bounds);
-        VariantClear(ref.u.v);
-        V_VT(ref.u.v) = VT_ARRAY|VT_VARIANT;
-        V_ARRAY(ref.u.v) = array;
+        VariantClear(v);
+        V_VT(v) = VT_ARRAY|VT_VARIANT;
+        V_ARRAY(v) = array;
         return S_OK;
     } else if(array->cDims != dim_cnt) {
         /* can't otherwise change the number of dimensions */
@@ -2398,13 +2413,13 @@ static void release_exec(exec_ctx_t *ctx)
             if(ctx->arrays[i])
                 SafeArrayDestroy(ctx->arrays[i]);
         }
-        heap_free(ctx->arrays);
+        free(ctx->arrays);
     }
 
     heap_pool_free(&ctx->heap);
-    heap_free(ctx->args);
-    heap_free(ctx->vars);
-    heap_free(ctx->stack);
+    free(ctx->args);
+    free(ctx->vars);
+    free(ctx->stack);
 }
 
 HRESULT exec_script(script_ctx_t *ctx, BOOL extern_caller, function_t *func, vbdisp_t *vbthis, DISPPARAMS *dp, VARIANT *res)
@@ -2427,7 +2442,7 @@ HRESULT exec_script(script_ctx_t *ctx, BOOL extern_caller, function_t *func, vbd
         VARIANT *v;
         unsigned i;
 
-        exec.args = heap_alloc_zero(func->arg_cnt * sizeof(VARIANT));
+        exec.args = calloc(func->arg_cnt, sizeof(VARIANT));
         if(!exec.args) {
             release_exec(&exec);
             return E_OUTOFMEMORY;
@@ -2454,7 +2469,7 @@ HRESULT exec_script(script_ctx_t *ctx, BOOL extern_caller, function_t *func, vbd
     }
 
     if(func->var_cnt) {
-        exec.vars = heap_alloc_zero(func->var_cnt * sizeof(VARIANT));
+        exec.vars = calloc(func->var_cnt, sizeof(VARIANT));
         if(!exec.vars) {
             release_exec(&exec);
             return E_OUTOFMEMORY;
@@ -2465,7 +2480,7 @@ HRESULT exec_script(script_ctx_t *ctx, BOOL extern_caller, function_t *func, vbd
 
     exec.stack_size = 16;
     exec.top = 0;
-    exec.stack = heap_alloc(exec.stack_size * sizeof(VARIANT));
+    exec.stack = malloc(exec.stack_size * sizeof(VARIANT));
     if(!exec.stack) {
         release_exec(&exec);
         return E_OUTOFMEMORY;
