@@ -106,6 +106,7 @@ struct connect
 struct netconn
 {
     struct list entry;
+    LONG refs;
     int socket;
     struct sockaddr_storage sockaddr;
     BOOL secure; /* SSL active on connection? */
@@ -255,7 +256,7 @@ enum fragment_type
 struct socket
 {
     struct object_header hdr;
-    struct request *request;
+    struct netconn *netconn;
     int keepalive_interval;
     unsigned int send_buffer_size;
     enum socket_state state;
@@ -275,6 +276,8 @@ struct socket
     unsigned int send_remaining_size;
     unsigned int bytes_in_send_frame_buffer;
     unsigned int client_buffer_offset;
+    char *read_buffer;
+    unsigned int bytes_in_read_buffer;
     SRWLOCK send_lock;
     volatile LONG pending_noncontrol_send;
     enum fragment_type sending_fragment_type;
@@ -370,7 +373,8 @@ void close_connection( struct request * ) DECLSPEC_HIDDEN;
 void init_queue( struct queue *queue ) DECLSPEC_HIDDEN;
 void stop_queue( struct queue * ) DECLSPEC_HIDDEN;
 
-void netconn_close( struct netconn * ) DECLSPEC_HIDDEN;
+void netconn_addref( struct netconn * ) DECLSPEC_HIDDEN;
+void netconn_release( struct netconn * ) DECLSPEC_HIDDEN;
 DWORD netconn_create( struct hostdata *, const struct sockaddr_storage *, int, struct netconn ** ) DECLSPEC_HIDDEN;
 void netconn_unload( void ) DECLSPEC_HIDDEN;
 ULONG netconn_query_data_available( struct netconn * ) DECLSPEC_HIDDEN;
@@ -398,16 +402,6 @@ DWORD process_header( struct request *, const WCHAR *, const WCHAR *, DWORD, BOO
 extern HRESULT WinHttpRequest_create( void ** ) DECLSPEC_HIDDEN;
 void release_typelib( void ) DECLSPEC_HIDDEN;
 
-static inline WCHAR *strdupW( const WCHAR *src )
-{
-    WCHAR *dst;
-
-    if (!src) return NULL;
-    dst = malloc( (lstrlenW( src ) + 1) * sizeof(WCHAR) );
-    if (dst) lstrcpyW( dst, src );
-    return dst;
-}
-
 static inline WCHAR *strdupAW( const char *src )
 {
     WCHAR *dst = NULL;
