@@ -125,7 +125,9 @@
     SYSCALL_ENTRY( NtUserDestroyWindow ) \
     SYSCALL_ENTRY( NtUserDisableThreadIme ) \
     SYSCALL_ENTRY( NtUserDispatchMessage ) \
+    SYSCALL_ENTRY( NtUserDisplayConfigGetDeviceInfo ) \
     SYSCALL_ENTRY( NtUserDragDetect ) \
+    SYSCALL_ENTRY( NtUserDragObject ) \
     SYSCALL_ENTRY( NtUserDrawIconEx ) \
     SYSCALL_ENTRY( NtUserEmptyClipboard ) \
     SYSCALL_ENTRY( NtUserEnableMenuItem ) \
