@@ -34,6 +34,7 @@ enum
     NtUserCallWinProc,
     NtUserCallWindowsHook,
     NtUserCopyImage,
+    NtUserDrawNonClientButton,
     NtUserDrawScrollBar,
     NtUserDrawText,
     NtUserFreeCachedClipboardData,
@@ -52,6 +53,8 @@ enum
     /* Vulkan support */
     NtUserCallVulkanDebugReportCallback,
     NtUserCallVulkanDebugUtilsCallback,
+    /* OpenGL support */
+    NtUserCallOpenGLDebugMessageCallback,
     /* Driver-specific callbacks */
     NtUserDriverCallbackFirst,
     NtUserDriverCallbackLast = NtUserDriverCallbackFirst + 9,
@@ -418,6 +421,17 @@ struct set_clipboard_params
     UINT   seqno;
 };
 
+/* NtUserNonClientButton params */
+struct draw_non_client_button_params
+{
+    HWND hwnd;
+    HDC hdc;
+    enum NONCLIENT_BUTTON_TYPE type;
+    RECT rect;
+    BOOL down;
+    BOOL grayed;
+};
+
 /* NtUserDrawScrollBar params */
 struct draw_scroll_bar_params
 {
