@@ -385,6 +385,13 @@ struct UiaWindowClosedEventArgs {
     int cRuntimeIdLen;
 };
 
+struct UiaFindParams {
+    int MaxDepth;
+    BOOL FindFirst;
+    BOOL ExcludeRoot;
+    struct UiaCondition *pFindCondition;
+};
+
 typedef SAFEARRAY * WINAPI UiaProviderCallback(HWND hwnd,enum ProviderType providerType);
 
 HRESULT WINAPI UiaGetReservedMixedAttributeValue(IUnknown **value);
@@ -409,6 +416,8 @@ HRESULT WINAPI UiaGetUpdatedCache(HUIANODE huianode, struct UiaCacheRequest *cac
         struct UiaCondition *normalize_cond, SAFEARRAY **out_req, BSTR *tree_struct);
 HRESULT WINAPI UiaNavigate(HUIANODE huianode, enum NavigateDirection dir, struct UiaCondition *nav_condition,
         struct UiaCacheRequest *cache_req, SAFEARRAY **out_req, BSTR *tree_struct);
+HRESULT WINAPI UiaFind(HUIANODE huianode, struct UiaFindParams *find_params, struct UiaCacheRequest *cache_req, SAFEARRAY **out_req,
+        SAFEARRAY **out_offsets, SAFEARRAY **out_tree_structs);
 
 #ifdef __cplusplus
 }
