@@ -21,13 +21,14 @@
 #ifndef __WINE_WINE_UNIXLIB_H
 #define __WINE_WINE_UNIXLIB_H
 
-typedef NTSTATUS (*unixlib_entry_t)( void *args );
 typedef UINT64 unixlib_handle_t;
 
 extern NTSTATUS WINAPI __wine_unix_call( unixlib_handle_t handle, unsigned int code, void *args );
 
 #ifdef WINE_UNIX_LIB
 
+typedef NTSTATUS (*unixlib_entry_t)( void *args );
+
 /* some useful helpers from ntdll */
 extern const char *ntdll_get_build_dir(void);
 extern const char *ntdll_get_data_dir(void);
@@ -264,6 +265,14 @@ static inline ULONG ntdll_wcstoul( const WCHAR *s, WCHAR **end, int base )
 #define wcstol(str,e,b)    ntdll_wcstol(str,e,b)
 #define wcstoul(str,e,b)   ntdll_wcstoul(str,e,b)
 
+#else /* WINE_UNIX_LIB */
+
+extern unixlib_handle_t __wine_unixlib_handle DECLSPEC_HIDDEN;
+extern NTSTATUS (WINAPI *__wine_unix_call_ptr)( unixlib_handle_t, unsigned int, void * ) DECLSPEC_HIDDEN;
+extern NTSTATUS WINAPI __wine_init_unix_call(void) DECLSPEC_HIDDEN;
+
+#define WINE_UNIX_CALL(code,args) __wine_unix_call_ptr( __wine_unixlib_handle, (code), (args) )
+
 #endif /* WINE_UNIX_LIB */
 
 #endif  /* __WINE_WINE_UNIXLIB_H */
